﻿# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

# <pep8 compliant>

bl_info = {
	"name": "Export Fusion Camera Animation",
	"author": "michael vorberg",
	"version": (0, 1),
	"blender": (2, 57, 0),
	"location": "File > Export > Fusion Cameras (.comp)",
	"description": "Export Fusion Cameras (.comp)",
	"warning": "",
	"category": "Import-Export",
}


import bpy
import math

def write_cameras(context, filepath, frame_start, frame_end, only_selected=False):

	data_attrs = (
		'lens',
		'shift_x',
		'shift_y',
		'dof_distance',
		'clip_start',
		'clip_end',
		'draw_size',
		)

	obj_attrs = (
		'hide_render',
		)

	fw = open(filepath, 'w')

	scene = bpy.context.scene

	cameras = []

	for obj in scene.objects:
		if only_selected and not obj.select:
			continue
		if obj.type != 'CAMERA':
			continue

		cameras.append((obj, obj.data))

	frame_range = range(frame_start, frame_end + 1)

	for obj, obj_data in cameras:
		#fw.write("data = bpy.data.cameras.new(%r)\n" % obj.name)
		#for attr in data_attrs:
		 #	 fw.write("data.%s = %s\n" % (attr, repr(getattr(obj_data, attr))))

		#fw.write("obj = bpy.data.objects.new(%r, data)\n" % obj.name)

		#for attr in obj_attrs:
		 #	 fw.write("obj.%s = %s\n" % (attr, repr(getattr(obj, attr))))

		#fw.write("scene.objects.link(obj)\n")
		#fw.write("cameras[%r] = obj\n" % obj.name)
		#fw.write("\n")

		fw.write( "Composition {\n"
			"	CurrentTime = 0,\n")
		fw.write(	"	RenderRange = { %d, %d, },\n"%(frame_start, frame_end+1))
		fw.write(	"	GlobalRange = { %d, %d, },\n"%(frame_start, frame_end+1))
		fw.write(	"Tools = {\n"
			"		Camera3D_Blender = Camera3D {\n"
			"			CtrlWZoom = false,\n"
			"			Inputs = {\n"
			"				[\"PerspNearClip\"] = Input {\n"
			"					SourceOp = \"Camera3D1PerspNearClip\",\n"
			"					Source = \"Value\",\n"
			"				},\n"
			"				[\"PerspFarClip\"] = Input {\n"
			"					SourceOp = \"Camera3D1PerspFarClip\",\n"
			"					Source = \"Value\",\n"
			"				},\n"
			"				[\"FLength\"] = Input {\n"
			"					SourceOp = \"Camera3D1LensFlength\",\n"
			"					Source = \"Value\",\n"
			"				},\n"
			"				[\"Transform3DOp.Translate.X\"] = Input {\n"
			"					SourceOp = \"Camera3D1XOffset1\",\n"
			"					Source = \"Value\",\n"
			"				},\n"
			"				[\"Transform3DOp.Translate.Y\"] = Input {\n"
			"					SourceOp = \"Camera3D1YOffset1\",\n"
			"					Source = \"Value\",\n"
			"				},\n"
			"				[\"Transform3DOp.Translate.Z\"] = Input {\n"
			"					SourceOp = \"Camera3D1ZOffset1\",\n"
			"					Source = \"Value\",\n"
			"				},\n"
			"				[\"Transform3DOp.Rotate.X\"] = Input {\n"
			"					SourceOp = \"Camera3D1XRotation\",\n"
			"					Source = \"Value\",\n"
			"				},\n"
			"				[\"Transform3DOp.Rotate.Y\"] = Input {\n"
			"					SourceOp = \"Camera3D1YRotation\",\n"
			"					Source = \"Value\",\n"
			"				},\n"
			"				[\"Transform3DOp.Rotate.Z\"] = Input {\n"
			"					SourceOp = \"Camera3D1ZRotation\",\n"
			"					Source = \"Value\",\n"
			"				},\n")
			
		#fw.write("				FLength = Input { Value = %s, },\n" % obj_data.lens)
		fw.write("				[\"Stereo.Mode\"] = Input { Value = FuID { \"Mono\", }, },\n"
			"				FilmBack = Input { Value = 1, },\n"
			"				FilmGate = Input { Value = FuID { \"User\", }, },\n")
		ap_w = 	float(obj_data.sensor_width)/25.1
		fw.write("				ApertureW = Input { Value = %s, },\n" %str(ap_w))				
		ap_h = ap_w/(scene.render.resolution_x/scene.render.resolution_y)
		fw.write("				ApertureH = Input { Value = %s, },\n" %str(ap_h))
		#fw.write("				LensShiftX = Input { Value = %s, },\n" %obj_data.shift_x)
		#fw.write("				LensShiftY = Input { Value = %s, },\n" %obj_data.shift_y)
		#fw.write("				PerspNearClip = Input { Value = %s, },\n" %obj_data.clip_start)
		#fw.write("				PerspFarClip = Input { Value = %s, },\n" %obj_data.clip_end)
		fw.write("				[\"SurfacePlaneInputs.ObjectID.ObjectID\"] = Input { Value = 1, },\n"
			"				[\"MtlStdInputs.MaterialID\"] = Input { Value = 1, },\n"
			"			},\n"
			"			ViewInfo = OperatorInfo { Pos = { 449, 147, }, },\n"
			"		},\n"
			)

		fw.write("		Camera3D1XOffset1 = BezierSpline {\n"
			"		SplineColor = { Red = 250, Green = 205, Blue = 49, },\n"
			"		NameSet = true,\n"
			"		KeyFrames = {\n")
	for f in frame_range:
		scene.frame_set(f)
		#fw.write("# new frame\n")
		#fw.write("scene.frame_set(%d + frame)\n" % f)

		for obj, obj_data in cameras:
			#fw.write("obj = cameras['%s']\n" % obj.name)

			matrix = obj.matrix_world.copy()
			fw.write("	[%d]" % f)
			fw.write("= { %r, Flags = { Linear = true, }, },\n"  %matrix.to_translation()[0])
			#fw.write("obj.location = %r, %r, %r\n" % matrix.to_translation()[:])
			#fw.write("obj.scale = %r, %r, %r\n" % matrix.to_scale()[:])
			#fw.write("obj.rotation_euler = %r, %r, %r\n" % matrix.to_euler()[:])

			#fw.write("obj.keyframe_insert('location')\n")
			#fw.write("obj.keyframe_insert('scale')\n")
			#fw.write("obj.keyframe_insert('rotation_euler')\n")

			# only key the angle
			#fw.write("data = obj.data\n")
			#fw.write("data.lens = %s\n" % obj_data.lens)
			#fw.write("data.keyframe_insert('lens')\n")

			#fw.write("\n")

	fw.write("  },\n"
	   "},\n")
	fw.write("		Camera3D1YOffset1 = BezierSpline {\n"
			"		SplineColor = { Red = 250, Green = 205, Blue = 49, },\n"
			"		NameSet = true,\n"
			"		KeyFrames = {\n")
	for f in frame_range:
		scene.frame_set(f)
		#fw.write("# new frame\n")
		#fw.write("scene.frame_set(%d + frame)\n" % f)

		for obj, obj_data in cameras:
			#fw.write("obj = cameras['%s']\n" % obj.name)

			matrix = obj.matrix_world.copy()
			fw.write("	[%d]" % f)
			fw.write("= { %r, Flags = { Linear = true, }, },\n"  %matrix.to_translation()[1])
	fw.write("  },\n"
	   "},\n")
	   
	fw.write("		Camera3D1ZOffset1 = BezierSpline {\n"
			"		SplineColor = { Red = 250, Green = 205, Blue = 49, },\n"
			"		NameSet = true,\n"
			"		KeyFrames = {\n")
	for f in frame_range:
		scene.frame_set(f)
		#fw.write("# new frame\n")
		#fw.write("scene.frame_set(%d + frame)\n" % f)

		for obj, obj_data in cameras:
			#fw.write("obj = cameras['%s']\n" % obj.name)

			matrix = obj.matrix_world.copy()
			fw.write("	[%d]" % f)
			fw.write("= { %r, Flags = { Linear = true, }, },\n"  %matrix.to_translation()[2])
	
	fw.write("  },\n"
	   "},\n")
	 
	fw.write("		Camera3D1XRotation = BezierSpline {\n"
			"		SplineColor = { Red = 250, Green = 205, Blue = 49, },\n"
			"		NameSet = true,\n"
			"		KeyFrames = {\n")
	for f in frame_range:
		scene.frame_set(f)
	
		for obj, obj_data in cameras:
	
			matrix = obj.matrix_world.copy()
			fw.write("	[%d]" % f)
			fw.write("= { %r, Flags = { Linear = true, }, },\n"  %math.degrees(matrix.to_euler()[0]))
	
	fw.write("  },\n"
	   "},\n")
	   
	fw.write("		Camera3D1YRotation = BezierSpline {\n"
			"		SplineColor = { Red = 250, Green = 205, Blue = 49, },\n"
			"		NameSet = true,\n"
			"		KeyFrames = {\n")
	for f in frame_range:
		scene.frame_set(f)
	
		for obj, obj_data in cameras:
	
			matrix = obj.matrix_world.copy()
			fw.write("	[%d]" % f)
			fw.write("= { %r, Flags = { Linear = true, }, },\n"  %math.degrees(matrix.to_euler()[1]))
	
	fw.write("  },\n"
	   "},\n")
	
	fw.write("		Camera3D1ZRotation = BezierSpline {\n"
			"		SplineColor = { Red = 250, Green = 205, Blue = 49, },\n"
			"		NameSet = true,\n"
			"		KeyFrames = {\n")
	for f in frame_range:
		scene.frame_set(f)
	
		for obj, obj_data in cameras:
	
			matrix = obj.matrix_world.copy()
			fw.write("	[%d] = { %r, Flags = { Linear = true, }, },\n"  %(f, math.degrees(matrix.to_euler()[2])))
	
	fw.write("  },\n"
	   "},\n")
	   
	fw.write("		Camera3D1LensFlength = BezierSpline {\n"
			"		SplineColor = { Red = 250, Green = 205, Blue = 49, },\n"
			"		NameSet = true,\n"
			"		KeyFrames = {\n")
	for f in frame_range:
		scene.frame_set(f)
	
		for obj, obj_data in cameras:
	
			matrix = obj.matrix_world.copy()
			fw.write("	[%d] = { %s, Flags = { Linear = true, }, },\n"  %(f, obj_data.lens))
	
	fw.write("  },\n"
	   "},\n")
	   
	fw.write("		Camera3D1PerspNearClip = BezierSpline {\n"
			"		SplineColor = { Red = 250, Green = 205, Blue = 49, },\n"
			"		NameSet = true,\n"
			"		KeyFrames = {\n")
	for f in frame_range:
		scene.frame_set(f)
	
		for obj, obj_data in cameras:
	
			matrix = obj.matrix_world.copy()
			fw.write("	[%d] = { %s, Flags = { Linear = true, }, },\n"  %(f, obj_data.clip_start))
	
	fw.write("  },\n"
	   "},\n")
	   
	fw.write("		Camera3D1PerspFarClip = BezierSpline {\n"
			"		SplineColor = { Red = 250, Green = 205, Blue = 49, },\n"
			"		NameSet = true,\n"
			"		KeyFrames = {\n")
	for f in frame_range:
		scene.frame_set(f)
	
		for obj, obj_data in cameras:
	
			matrix = obj.matrix_world.copy()
			fw.write("	[%d] = { %s, Flags = { Linear = true, }, },\n"  %(f, obj_data.clip_end))
	
	fw.write("  },\n"
	   "},\n")
	 
	fw.write("Transform3D_Blender = Transform3D {\n"
		"	CtrlWZoom = false,\n"
		"	Inputs = {\n"
		"		SceneInput = Input {\n"
		"			SourceOp = \"Camera3D_Blender\",\n"
		"			Source = \"Output\",\n"
		"		},\n"
		"		[\"Transform3DOp.Rotate.X\"] = Input { Value = -90, },\n"
		"	},\n"
		"},\n")
	
	#close composition
	fw.write("  },\n"
	   "}\n")

	fw.close()

from bpy.props import StringProperty, IntProperty, BoolProperty
from bpy_extras.io_utils import ExportHelper


class CameraExporter(bpy.types.Operator, ExportHelper):
	"""Save a python script which re-creates cameras and markers elsewhere"""
	bl_idname = "export_animation_fusion.cameras"
	bl_label = "Export Fusion Cameras"

	filename_ext = ".comp"
	filter_glob = StringProperty(default="*.comp", options={'HIDDEN'})

	frame_start = IntProperty(name="Start Frame",
			description="Start frame for export",
			default=1, min=1, max=300000)
	frame_end = IntProperty(name="End Frame",
			description="End frame for export",
			default=250, min=1, max=300000)
	only_selected = BoolProperty(name="Only Selected",
			default=True)

	def execute(self, context):
		write_cameras(context, self.filepath, self.frame_start, self.frame_end, self.only_selected)
		return {'FINISHED'}

	def invoke(self, context, event):
		self.frame_start = context.scene.frame_start
		self.frame_end = context.scene.frame_end

		wm = context.window_manager
		wm.fileselect_add(self)
		return {'RUNNING_MODAL'}


def menu_export(self, context):
	import os
	default_path = os.path.splitext(bpy.data.filepath)[0] + ".comp"
	self.layout.operator(CameraExporter.bl_idname, text="Fusion Cameras (.comp)").filepath = default_path


def register():
	bpy.utils.register_module(__name__)

	bpy.types.INFO_MT_file_export.append(menu_export)


def unregister():
	bpy.utils.unregister_module(__name__)

	bpy.types.INFO_MT_file_export.remove(menu_export)


if __name__ == "__main__":
	register()
